// Underworld Random Map by xTrinity (AzN)
// Version FINAL1
//
// Bugs:
// *Bad Generation:	This bug occurs when the lava and tree constraints are incorrectly generated which will leave
//			wide gaps of flat and unpopulated terrain. Or it can even surround a player in lava.
//			This is a bug with the generator so I cannot fix it. If this does occur just restart...
// *Objects in Lava:	This bug occurs when the "avoidImpassableLand" tag is not parsed or parsed incorrectly.
//			This will leave objects stuck in the lava which are un retrievable. This bug is a problem
//			of the generator and cannot be fixed by me. Fire and Fog are purposely put in Lava.
// *Instant Loss:	Again a bug with the generator. This bug occurs when the "avoidBuildings" or similar tags are
//			incorrectly parsed by the generator. This leaves thick forests or lava inside your starting
//			point which obstructs the construction of your Town Center causing instant loss. Again I
//			cannot fix this.
// *Greenish Lava:	This is not technically a bug. With the Eclipse lighting I used with this script it causes
//			items on the map to use different colored lighting. I used Lighting to create the coloring
//			of what the Underworld should look like although there may be side effects. Its better then
//			Night lighting because then the Lava is completely green. And its better then no lighting because
//			the Underworld would look its brightly lit by a sun. So if your really annoyed then either change 
//			your Brightness/Contrast or make me a better LightSet.
// *Burning Trees:	Burning Trees will not allow you to collect from it. Also some Burning Trees don't have Flame.
//			That is a game bug. Otherwise, its just how the game is!
// *Lightning Clouds:	Sometimes Lightning Clouds will cover what your trying to see. This is the natural behavior of
//			the clouds. Just wait until they move out of the way. This can also be used to your advantage
//			as you can build within the Clouds and the enemy wont know what's under ;-).
// *Fires:		Same as Lightning Clouds except these never move out of the way. If your having trouble seeing
//			rotate the camera.
//
// Submit Bugs to: AznKnightmare@hotmail.com
//

void main(void)

{
   rmSetStatusText("This first begins my 0wnage...",0.01);


   int playerTiles=7500;
   if(cMapSize == 1)
   {
      playerTiles = 9750;
      rmEchoInfo("Large map");
   }
   int size=2.0*sqrt(cNumberNonGaiaPlayers*playerTiles/0.9);
   rmEchoInfo("Map size="+size+"m x "+size+"m");
   rmSetMapSize(size, size);
   rmSetSeaLevel(0.0);
   if(rmRandFloat(0,1)<0.5)
   {
      rmTerrainInitialize("hadesbuildable2");
   }
   else
   {
      rmTerrainInitialize("hadesbuildable1");
   }
   rmSetLightingSet("eclipse");

   int classPlayer=rmDefineClass("player");
   rmDefineClass("corner");
   rmDefineClass("starting settlement");

   int edgeConstraint=rmCreateBoxConstraint("edge of map", rmXTilesToFraction(8), rmZTilesToFraction(8), 1.0-rmXTilesToFraction(8), 1.0-rmZTilesToFraction(8));
   int playerConstraint=rmCreateClassDistanceConstraint("stay away from players", classPlayer, size*0.05);
   int farAvoidSettlement=rmCreateTypeDistanceConstraint("objects avoid TC by long distance", "AbstractSettlement", 50.0);
   int farStartingSettleConstraint=rmCreateClassDistanceConstraint("objects avoid player TCs", rmClassID("starting settlement"), 70.0);
   int avoidTower=rmCreateTypeDistanceConstraint("towers avoid towers", "tower", 25.0);
   int avoidBuildings=rmCreateTypeDistanceConstraint("avoid buildings", "Building", 20.0);
   int avoidGold=rmCreateTypeDistanceConstraint("avoid gold", "gold", 30.0);
   int avoidRandomTree=rmCreateTypeDistanceConstraint("random tree", "all", 4.0);
   int forestSettleConstraint=rmCreateClassDistanceConstraint("forest settle", rmClassID("starting settlement"), 20.0);
   int avoidHerdable=rmCreateTypeDistanceConstraint("avoid herdable", "herdable", 20.0);
   int avoidFood=rmCreateTypeDistanceConstraint("avoid other food sources", "food", 5.0);
   int avoidPredator=rmCreateTypeDistanceConstraint("avoid predator", "animalPredator", 20.0);
   int avoidRelic=rmCreateTypeDistanceConstraint("relic vs relic", "relic", 70.0);
   int allObjConstraint=rmCreateTypeDistanceConstraint("all obj", "all", 5.0);
   int avoidImpassableLand=rmCreateTerrainDistanceConstraint("avoid impassable land", "land", false, 10.0);


   rmSetStatusText("Now I populate my 0wnage with n00bies...",0.10);


   int randomCloudID=rmCreateObjectDef("random cloud");
   rmAddObjectDefItem(randomCloudID, "Lightning Cloud", 1, 0.0);
   rmSetObjectDefMinDistance(randomCloudID, 0.0);
   rmSetObjectDefMaxDistance(randomCloudID, rmXFractionToMeters(0.5));

   int randomFireID=rmCreateObjectDef("random fire");
   rmAddObjectDefItem(randomFireID, "Underworld Explosion", 1, 15.0);
   rmSetObjectDefMinDistance(randomFireID, 0.0);
   rmSetObjectDefMaxDistance(randomFireID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(randomFireID, farStartingSettleConstraint);

   int randomFireMiniID=rmCreateObjectDef("random firemini");
   rmAddObjectDefItem(randomFireMiniID, "Hades Fire", 1, 10.0);
   rmSetObjectDefMinDistance(randomFireMiniID, 0.0);
   rmSetObjectDefMaxDistance(randomFireMiniID, rmXFractionToMeters(0.5));

   int startingSettlementID=rmCreateObjectDef("starting settlement");
   rmAddObjectDefItem(startingSettlementID, "Settlement Level 1", 1, 0.0);
   rmAddObjectDefToClass(startingSettlementID, rmClassID("starting settlement"));
   rmSetObjectDefMinDistance(startingSettlementID, 0.0);
   rmSetObjectDefMaxDistance(startingSettlementID, 0.0);

   int startingTowerID=rmCreateObjectDef("Starting tower");
   rmAddObjectDefItem(startingTowerID, "Tower", 1, 0.0);
   rmSetObjectDefMinDistance(startingTowerID, 30.0);
   rmSetObjectDefMaxDistance(startingTowerID, 30.0);
   rmAddObjectDefConstraint(startingTowerID, avoidTower);
   rmAddObjectDefConstraint(startingTowerID, avoidImpassableLand);
   
   int startingGoldID=rmCreateObjectDef("Starting gold");
   float goldStarting=rmRandFloat(0.0, 1.0);
   if(goldStarting<0.5)
      rmAddObjectDefItem(startingGoldID, "Gold Mine", 1, 0.0);
   else
      rmAddObjectDefItem(startingGoldID, "Gold mine small", 1, 0.0);
   rmSetObjectDefMinDistance(startingGoldID, 28.0);
   rmSetObjectDefMaxDistance(startingGoldID, 28.0);
   rmAddObjectDefConstraint(startingGoldID, avoidImpassableLand);

   int closeGoatsID=rmCreateObjectDef("close goats");
   float goatsClose=rmRandFloat(0.0, 1.0);
   if(goatsClose<0.5)
      rmAddObjectDefItem(closeGoatsID, "Goat", 3, 4.0);
   else
      rmAddObjectDefItem(closeGoatsID, "Goat", 4, 4.0);
   rmSetObjectDefMinDistance(closeGoatsID, 20.0);
   rmSetObjectDefMaxDistance(closeGoatsID, 20.0);
   rmAddObjectDefConstraint(closeGoatsID, avoidFood);
   rmAddObjectDefConstraint(closeGoatsID, avoidImpassableLand);
   
   int closeBerriesID=rmCreateObjectDef("close berries");
   float berriesClose=rmRandFloat(0.0, 1.0);
   if(berriesClose<0.5)
      rmAddObjectDefItem(closeBerriesID, "Berry bush", 3, 4.0);
   else
      rmAddObjectDefItem(closeBerriesID, "Berry bush", 4, 4.0);
   rmSetObjectDefMinDistance(closeBerriesID, 20.0);
   rmSetObjectDefMaxDistance(closeBerriesID, 20.0);
   rmAddObjectDefConstraint(closeBerriesID, avoidFood);
   rmAddObjectDefConstraint(closeBerriesID, avoidImpassableLand);
   
   int stragglerTreeID=rmCreateObjectDef("straggler tree");
   float treeStraggler=rmRandFloat(0.0, 1.0);
   if(treeStraggler<0.5)
      rmAddObjectDefItem(stragglerTreeID, "Pine dead", 2, 5.0);
   else
      rmAddObjectDefItem(stragglerTreeID, "Pine dead", 3, 5.0);
   rmSetObjectDefMinDistance(stragglerTreeID, 10.0);
   rmSetObjectDefMaxDistance(stragglerTreeID, 15.0);


   int mediumGoldID=rmCreateObjectDef("medium gold");
   float goldMedium=rmRandFloat(0.0, 1.0);
   if(goldMedium<0.5)
      rmAddObjectDefItem(mediumGoldID, "Gold mine", 1, 0.0);
   else
      rmAddObjectDefItem(mediumGoldID, "Gold mine small", 1, 0.0);
   rmSetObjectDefMinDistance(mediumGoldID, 40.0);
   rmSetObjectDefMaxDistance(mediumGoldID, 50.0);
   rmAddObjectDefConstraint(mediumGoldID, avoidGold);
   rmAddObjectDefConstraint(mediumGoldID, avoidImpassableLand);
  
   int mediumGoatsID=rmCreateObjectDef("medium goats");
   float goatsMedium=rmRandFloat(0.0, 1.0);
   if(goatsMedium<0.5)
      rmAddObjectDefItem(mediumGoatsID, "Goat", 3, 4.0);
   else
      rmAddObjectDefItem(mediumGoatsID, "Goat", 4, 4.0);
   rmSetObjectDefMinDistance(mediumGoatsID, 40.0);
   rmSetObjectDefMaxDistance(mediumGoatsID, 50.0);
   rmAddObjectDefConstraint(mediumGoatsID, avoidHerdable);
   rmAddObjectDefConstraint(mediumGoatsID, avoidImpassableLand);


   int farSettlementID=rmCreateObjectDef("far settlement");
   rmAddObjectDefItem(farSettlementID, "Settlement", 1, 0.0);
   rmSetObjectDefMinDistance(farSettlementID, 80.0);
   rmSetObjectDefMaxDistance(farSettlementID, 100.0);
   rmAddObjectDefConstraint(farSettlementID, farAvoidSettlement);
   rmAddObjectDefConstraint(farSettlementID, avoidImpassableLand);

   int farGoldID=rmCreateObjectDef("far gold");
   float goldFar=rmRandFloat(0.0, 1.0);
   if(goldFar<0.5)
      rmAddObjectDefItem(farGoldID, "Gold mine", 1, 0.0);
   else
      rmAddObjectDefItem(farGoldID, "Gold mine small", 1, 0.0);
   rmSetObjectDefMinDistance(farGoldID, 50.0);
   rmSetObjectDefMaxDistance(farGoldID, 100.0);
   rmAddObjectDefConstraint(farGoldID, avoidGold);
   rmAddObjectDefConstraint(farGoldID, farStartingSettleConstraint);
   rmAddObjectDefConstraint(farGoldID, avoidImpassableLand);
   
   int farPredatorID=rmCreateObjectDef("far predator");
   float predatorFar=rmRandFloat(0.0, 1.0);
   if(predatorFar<0.5)
      rmAddObjectDefItem(farPredatorID, "Serpent", 2, 4.0);
   else
      rmAddObjectDefItem(farPredatorID, "Shade of erebus", 2, 4.0);
   rmSetObjectDefMinDistance(farPredatorID, 50.0);
   rmSetObjectDefMaxDistance(farPredatorID, 75.0);
   rmAddObjectDefConstraint(farPredatorID, avoidPredator);
   rmAddObjectDefConstraint(farPredatorID, farStartingSettleConstraint);
   rmAddObjectDefConstraint(farPredatorID, avoidImpassableLand);

   int randomBurnTreeID=rmCreateObjectDef("random tree");
   rmAddObjectDefItem(randomBurnTreeID, "Pine dead burning", 1, 0.0);
   rmSetObjectDefMinDistance(randomBurnTreeID, 0.0);
   rmSetObjectDefMaxDistance(randomBurnTreeID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(randomBurnTreeID, avoidRandomTree);
   
   int relicID=rmCreateObjectDef("relic");
   rmAddObjectDefItem(relicID, "Relic", 1, 0.0);
   rmSetObjectDefMinDistance(relicID, 50.0);
   rmSetObjectDefMaxDistance(relicID, 150.0);
   rmAddObjectDefConstraint(relicID, avoidRelic);
   rmAddObjectDefConstraint(relicID, avoidImpassableLand);


   rmSetStatusText("And then I create land for the n00bs...",0.20);


   if(cNumberNonGaiaPlayers < 10)
   {
      rmSetTeamSpacingModifier(0.75);
      rmPlacePlayersCircular(0.3, 0.4, rmDegreesToRadians(4.0));
   }
   else
   {
      rmSetTeamSpacingModifier(0.85);
      rmPlacePlayersCircular(0.35, 0.45, rmDegreesToRadians(4.0));
   }

   float playerFraction=rmAreaTilesToFraction(4000);
   for(i=1; <cNumberPlayers)
   {
      int id=rmCreateArea("Player"+i);
      rmSetPlayerArea(i, id);
      rmSetAreaSize(id, 0.9*playerFraction, 1.0*playerFraction);
      rmAddAreaToClass(id, classPlayer);
      rmSetAreaMinBlobs(id, 1);
      rmSetAreaMaxBlobs(id, 5);
      rmSetAreaMinBlobDistance(id, 15.0);
      rmSetAreaMaxBlobDistance(id, 50.0);
      rmSetAreaCoherence(id, 0.0);
      rmAddAreaConstraint(id, playerConstraint);
      rmSetAreaLocPlayer(id, i);
      if(rmRandFloat(0,1)<0.5)
      {
         rmSetAreaTerrainType(id, "hadesbuildable2");
         rmAddAreaTerrainLayer(id, "hadesbuildable1", 0, 1);
      }
      else
      {
         rmSetAreaTerrainType(id, "hadesbuildable1");
         rmAddAreaTerrainLayer(id, "hadesbuildable2", 0, 1);
      } 
   }

   rmBuildAllAreas();

   for(i=1; <cNumberPlayers)
   {
      int id2=rmCreateArea("Player inner"+i, rmAreaID("Player"+i));
      rmSetAreaSize(id2, rmAreaTilesToFraction(400), rmAreaTilesToFraction(600));
      rmSetAreaLocPlayer(id2, i);
      if(rmRandFloat(0,1)<0.5)
      {
         rmSetAreaTerrainType(id2, "hadesbuildable2");
         rmAddAreaTerrainLayer(id2, "hadesbuildable1", 0, 1);
      }
      else
      {
         rmSetAreaTerrainType(id2, "hadesbuildable1");
         rmAddAreaTerrainLayer(id2, "hadesbuildable2", 0, 1);
      } 
      rmSetAreaMinBlobs(id2, 1);
      rmSetAreaMaxBlobs(id2, 5);
      rmSetAreaMinBlobDistance(id2, 10.0);
      rmSetAreaMaxBlobDistance(id2, 50.0);
      rmSetAreaCoherence(id2, 0.0);
      rmBuildArea(id2);
   }


   rmSetStatusText("So then I place their n00bie little buildings...",0.30);
   rmSetStatusText("And something their n00bie little mouths to eat...",0.40);
   rmSetStatusText("So began their n00bie little lives...",0.50);


   rmPlaceObjectDefAtLoc(randomFireID, 0, 0.5, 0.5, 5*cNumberNonGaiaPlayers);
   rmPlaceObjectDefAtLoc(randomCloudID, 0, 0.5, 0.5, 5*cNumberNonGaiaPlayers);
   rmPlaceObjectDefAtLoc(randomFireMiniID, 0, 0.5, 0.5, 15*cNumberNonGaiaPlayers);
   rmPlaceObjectDefPerPlayer(startingSettlementID, true);
   rmPlaceObjectDefPerPlayer(startingTowerID, true, rmRandInt(4, 5));
   rmPlaceObjectDefPerPlayer(startingGoldID, true, rmRandInt(1, 2));
   rmPlaceObjectDefPerPlayer(closeGoatsID, true, rmRandInt(1, 2));
   rmPlaceObjectDefPerPlayer(closeBerriesID, false, rmRandInt(1, 2));
   rmPlaceObjectDefPerPlayer(stragglerTreeID, false, rmRandInt(3, 5));
   rmPlaceObjectDefPerPlayer(mediumGoldID, false, rmRandInt(2, 3));
   rmPlaceObjectDefPerPlayer(mediumGoatsID, false, rmRandInt(1, 2));
   rmPlaceObjectDefPerPlayer(farSettlementID, false, rmRandInt(2, 3));
   rmPlaceObjectDefPerPlayer(farGoldID, false, rmRandInt(2, 4));
   rmPlaceObjectDefPerPlayer(farPredatorID, false, rmRandInt(1, 2));
   rmPlaceObjectDefAtLoc(randomBurnTreeID, 0, 0.5, 0.5, 15*cNumberNonGaiaPlayers);
   rmPlaceObjectDefAtLoc(relicID, 0, 0.5, 0.5, 5*cNumberNonGaiaPlayers);


   rmSetStatusText("Now I populate their n00bie little world with hills...",0.60);

   int classLava=rmDefineClass("lava");
   int numTries=30*cNumberNonGaiaPlayers;
   int failCount=0;
   for(i=0; <numTries)
   {
      int id3=rmCreateArea("Elev"+i);
      rmSetAreaSize(id3, rmAreaTilesToFraction(15), rmAreaTilesToFraction(120));
      rmSetAreaLocation(id3, rmRandFloat(0.0, 1.0), rmRandFloat(0.0, 1.0));
      rmSetAreaWarnFailure(id3, false);
      rmAddAreaConstraint(id3, avoidBuildings);
      rmSetAreaBaseHeight(id3, rmRandFloat(4.0, 5.0));
      if(rmRandFloat(0,1)<0.5)
      {
         rmSetAreaTerrainType(id3, "Hades4");
         rmAddAreaTerrainLayer(id3, "Hades3", 0, 1);
      }
      else
      {
         rmSetAreaTerrainType(id3, "Hades7");
         rmAddAreaTerrainLayer(id3, "Hades5", 0, 1);
      } 
      rmSetAreaMinBlobs(id3, 3);
      rmSetAreaMaxBlobs(id3, 4);
      rmSetAreaMinBlobDistance(id3, 10.0);
      rmSetAreaMaxBlobDistance(id3, 50.0);
      rmSetAreaCoherence(id3, 0.0);

      if(rmBuildArea(id3)==false)
      {
         failCount++;
         if(failCount==5)
            break;
      }
      else
         failCount=0;
   }

   numTries=70*cNumberNonGaiaPlayers;
   failCount=0;
   for(i=0; <numTries)
   {
      id3=rmCreateArea("Wrinkle"+i);
      rmSetAreaSize(id3, rmAreaTilesToFraction(15), rmAreaTilesToFraction(120));
      rmSetAreaLocation(id3, rmRandFloat(0.0, 1.0), rmRandFloat(0.0, 1.0));
      rmSetAreaWarnFailure(id3, false);
      rmSetAreaBaseHeight(id3, rmRandFloat(2.0, 3.0));
      rmSetAreaMinBlobs(id3, 2);
      rmSetAreaMaxBlobs(id3, 3);
      rmSetAreaMinBlobDistance(id3, 10.0);
      rmSetAreaMaxBlobDistance(id3, 20.0);
      rmSetAreaCoherence(id3, 0.0);

      if(rmBuildArea(id3)==false)
      {
         failCount++;
         if(failCount==5)
            break;
      }
      else
         failCount=0;
   }


   rmSetStatusText("At last, I grow their n00bie little trees for them...",0.60);


   int classForest=rmDefineClass("forest");
   int forestConstraint=rmCreateClassDistanceConstraint("forest v forest", rmClassID("forest"), 25.0);
   numTries=15*cNumberNonGaiaPlayers;
   failCount=0;
   for(i=0; <numTries)
   {
      int id4=rmCreateArea("Forest"+i);
      rmSetAreaSize(id4, rmAreaTilesToFraction(50), rmAreaTilesToFraction(100));
      rmSetAreaWarnFailure(id4, false);
      rmSetAreaForestType(id4, "Hades Forest");
      rmAddAreaConstraint(id4, forestSettleConstraint);
      rmAddAreaConstraint(id4, allObjConstraint);
      rmAddAreaConstraint(id4, forestConstraint);
      rmAddAreaToClass(id4, classForest);
      rmSetAreaMinBlobs(id4, 3);
      rmSetAreaMaxBlobs(id4, 7);
      rmSetAreaMinBlobDistance(id4, 16.0);
      rmSetAreaMaxBlobDistance(id4, 40.0);
      rmSetAreaCoherence(id4, 0.0);
      if(rmRandFloat(0.0, 1.0)<0.5)
         rmSetAreaBaseHeight(id4, rmRandFloat(3.0, 4.0));

      if(rmBuildArea(id4)==false)
      {
         failCount++;
         if(failCount==3)
            break;
      }
      else
         failCount=0;
   }


   rmSetStatusText("And so the n00bie humans flourished...",0.70);
   rmSetStatusText("To witness my 0wnage...",0.80);
   rmSetStatusText("And spread it upon other little n00bies...",0.90);
   rmSetStatusText("And so my 0wnage is complete...",1.00);


}  
// Underworld Series by xTrinity (AzN)
// The greatest scripter returns once again.... LOL